# Get significant climate QTLs and compare to flowering candidates and
# observed flowering QTLs

# Thomas Mitchell-Olds, revised 2020-11-30

library(tidyverse)
library(stringr)
library(readxl)


#-------------------------------------------------------------
# Input locations of significant SNPs from the climate PC experiment

half.width <- 100000 # 2 * half.width gives the window size around a SNP peak
PC.names <- c("PC2", "PC3", "PC4", "PC5")
num.PCs <- length(PC.names)
expt.folder <- str_c("_", PC.names)
infile.names <- str_c("Ref", expt.folder, ".txt_emmax.ps.mod")
# df.list <-  list("PC2", "PC3", "PC4", "PC5", 1:num.PCs, TRUE)

full.name <- rep(NA, num.PCs) # get file names for dfs
for(ii in(1:num.PCs)) { 
  full.name[ii] <- file.path(path.expand("~"),"FlrGWAS","bash", 
                             expt.folder[ii], infile.names[ii])
}

fix.df <- function(ii) { # function to read & process dataframe
  df <- read_tsv(full.name[ii])
  names(df) <- c("ChrNt", "EffectSize", "Pval", "Chr", "SNPpos")
  df <- df %>% mutate(Chr     = as.character(Chr),
                      SNPpos  = as.numeric(SNPpos),
                      Pval    = as.numeric(Pval),
                      log10P  = -1 * log10(Pval),
                      PC.env  = PC.names[ii]) %>% 
               select(Chr, SNPpos, log10P, PC.env)
  df <- df %>% filter(log10P >= 8.0)
  return(df)
}

df1 <- fix.df(1)
df2 <- fix.df(2)
df3 <- fix.df(3)
df4 <- fix.df(4)
df.all <- rbind(df1, df2, df3, df4) # Merge significant SNPs for each chromosome
df.all <- df.all %>% arrange(Chr, SNPpos)
glimpse(df.all)

#-------------------------------------------------------------

# Annotations of genes 
in_name <- "Bs_FLT_ortho.txt"  # These are only flowering time genes
infile <- file.path(path.expand("~"),"FlrGWAS","bash", "genome_data", in_name)
df.anot <- read_tsv(infile)
df.anot <- df.anot %>% select(Bs_gene, At_gene)
glimpse(df.anot)

# Locations of genes
in_name <- "Bstricta_278_v1.2.gene_exons.chr.gff3.xlsx"
infile <- file.path(path.expand("~"),"FlrGWAS","bash", "genome_data", in_name)
df.locs <- read_excel(infile, sheet = 1)
df.locs <- df.locs %>% mutate(Chr_Num = as.character(Chr_Num),
                              Chr_mid = round((Chr_start + Chr_end)/2), digits = 0)  %>% 
                       rename(Bs_gene = Name) %>% 
                       select(Bs_gene, Chr_Num, Chr_start, Chr_end, Chr_mid)
glimpse(df.locs)

df.merge <- left_join(df.anot, df.locs) %>% arrange(Chr_Num, Chr_mid)
# tail(df.merge)
df.merge <- na.omit(df.merge)
glimpse(df.merge) # These are only flowering time genes
# merged dataframe has Chr & Nt for each candidate flowering genes

for(jj in 1 : nrow(df.all)) {
  this.chr <- df.all$Chr[jj]
  this.Nt  <- df.all$SNPpos[jj]
  for(kk in 1 : nrow(df.merge)) {
    that.chr <- df.merge$Chr_Num[kk]
    if(this.chr == that.chr) {
      dist.Nt <- min(abs(this.Nt - df.merge$Chr_start[kk]), abs(this.Nt - df.merge$Chr_end[kk]))
      if(dist.Nt < half.width) {
        logP    <- df.all$log10P[jj]
        Bs.gene <- df.merge$Bs_gene[kk]   
        At.gene <- df.merge$At_gene[kk]
        PC.env  <- df.all$PC.env[jj]
        print(c(this.chr, this.Nt, logP, dist.Nt, PC.env, Bs.gene, At.gene))
      }
    }
  }
}











#-------------------------------------------------------------
