# Script to find pairs of environments for GxE analysis
# Find environments with the lowest correlations, excluding W3
# But also include E1W3
# Calculate plasticity as Xi - Yi

# Tom Mitchell-Olds, 2020-11-26

library(tidyverse)

# Input BLUPs for genotypes in 10 envirns
file_in <- "LogFlr_BLUPs_10_Expts.txt" 
in_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_in)
df.blups <- read_tsv(in_name)

# Simplify col names
df.BLUPs.merged <- df.blups %>% select(-ID)
env.names <- c("E1", "W1", "W2", "W3", "W4", "B1", "B3", "B4", "B5", "B6")
names(df.BLUPs.merged) <- env.names
dim(df.BLUPs.merged) # Number of genotypes x number of environments
# df.BLUPs.merged does not include RefPop ID column. This will be used
# to find the envirn pairs with lowest correlations
#----------------------------------------------------------

# Get corr matrix and pairwise correlations
(corrs <- cor(df.BLUPs.merged, use = "pairwise.complete.obs"))

x <- rep(NA, 45) # Empty vector for all pairs of correlations
# Get data frame with correlations
cell <-  0
for(row_i in 2:10) {
  for(col_j in 1:(row_i - 1)) {
    cell <- cell + 1
    x[cell] <- corrs[row_i, col_j]
    rho <-  c(cell, row_i, col_j, corrs[row_i, col_j])
    if(cell == 1) {
      Z <- rho
    } else {
      Z <-  rbind(Z, rho)
    }
  }
}

# Find the environment pairs that show lowest correlations
df <- as.tibble(Z)
names(df) <- c("cell", "row", "col", "corr")
df <- mutate(df, row.name = env.names[row],
                 col.name = env.names[col])
(df <- arrange(df, corr))
mean(df$corr)
# df is dataframe with pairwise correlations. 
# First use this to get the mean and range of corr values
# Second, use to find envirn pairs with lowest correlations (excluding W3)

#----------------------------------------------------------

# Does the correlation between first and second season expts differ from the rest?
# Compare B1_B4 and B3_B6 to the rest.

temp <- df %>% arrange(col.name, row.name)
temp$drop <- FALSE
temp$drop <- (temp$row.name == "B4" & temp$col.name == "B1") |
             (temp$row.name == "B6" & temp$col.name == "B3")

temp$corr[2] # Correlation for B1_B4 -- first & second year
temp$corr[7] # Correlation for B3_B6 -- first & second year

min(temp$corr[1:43])  # minimum correlation (excluding B1_B4 & B3_B6)
max(temp$corr[1:43])  # maximum correlation (excluding B1_B4 & B3_B6)
mean(temp$corr[1:43]) # mean correlation (excluding B1_B4 & B3_B6)

# Conclusion: First vs. second seasons show similar correlations
# as the other pairwise correlations among experiments

#----------------------------------------------------------

# Only retain envirn pairs that don't include divergent environment W3
df <- filter(df, row.name != "W3" & col.name != "W3")
dim(df)
head(df, n = 10)

#----------------------------------------------------------

df.blups.genos <- df.blups 
geno.env.names <- c("ID", "E1", "W1", "W2", "W3", "W4", "B1", "B3", "B4", "B5", "B6")
names(df.blups.genos) <- geno.env.names
dim(df.blups.genos) # 1 + number of genotypes x number of environments

#----------------------------------------------------------
# Function to extract info from pairwise correlations

pairZ <-  function(df1, EnvX, EnvY) {
  df2 <- df1 %>% select("ID", EnvX, EnvY) %>% na.omit()
  names(df2) <- c("RefPop", "X", "Y")
  Ngenos <- nrow(df2) # number of genotypes
  corXY <- cor(df2$X, df2$Y) # value of their correlation
  # Plasticity will be calculated based on log flr diff btwn envirns
  df2 <- df2 %>% mutate(FD.diff = X - Y)
  df3 <- df2 %>% select(RefPop, FD.diff) %>% arrange(RefPop)
  Envs <- paste(EnvX, EnvY, sep="")
  names(df3) <-  c("RefPop", Envs)
  # Info for output
  result <- list("N" = Ngenos, "Corr" = corXY, "df2" = df2, "df3" = df3)
}

#----------------------------------------------------------

# Analyze diffs in Log flowering time in six environment pairs. 
# Five pairs show low corrections. Also include E1W3
E1W3 <- pairZ(df.blups.genos, "E1", "W3")
W1W2 <- pairZ(df.blups.genos, "W1", "W2")
B1W2 <- pairZ(df.blups.genos, "B1", "W2")
B1W1 <- pairZ(df.blups.genos, "B1", "W1")
W2E1 <- pairZ(df.blups.genos, "W2", "E1")
B4W4 <- pairZ(df.blups.genos, "B1", "W4")

# Output data for GWAS
file_out <- "E1W3.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(E1W3$df3, out_name)

file_out <- "W1W2.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(W1W2$df3, out_name)

file_out <- "B1W2.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(B1W2$df3, out_name)

file_out <- "B1W1.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(B1W1$df3, out_name)

file_out <- "W2E1.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(W2E1$df3, out_name)

file_out <- "B4W4.txt"
out_name <- file.path(path.expand("~"),"FlrGWAS", "output", file_out)
write_tsv(B4W4$df3, out_name)









