# This script examines patterns of pleiotropy for flowering time across
# environments for genome-wide larger-effect SNPs. In each environment we find
# SNPs that have largest absolute effect size. For each SNP we count the number
# of environments with data, and choose the top 100 SNPs having data in at least
# MinNumEnvs environments.
# Then perform permutation test: does the average correlation among QTLs differ from the
# average correlation among SNPs?

# Thomas Mitchell-Olds, 2020-12-08

library(tidyverse)
library(stringr)

MinNumEnvs <- 6     # Data need in at least this many environments
NumSNPs    <- 3000  # Number of top SNPs analyzed per envirn
Too.Close  <- 3000  # Discard SNP if too close to previous SNP (dist in bp)

# Exclude regions affected by balancing selection (BLS):
# chr1: 14,423,807bp - 24,375,392bp
# chr5:    471,982bp -  3,400,076bp
# chr7:    451,506bp -  5,083,168bp
BLS <- data.frame(LG1 = c(1, 14423807, 24375392), 
                  LG5 = c(5, 471982, 3400076),
                  LG7 = c(7, 451506, 5083168))

NumNotNA <- function(rowvec) { 
  # For each SNP, count number of non-missing values & get related info
  rowvec <- rowvec[2:11] # columns with data from an environment
  rowNotNA <- rowvec[!is.na(rowvec)]
  NumWithData <- (length(rowNotNA))
  meanEffect  <- mean(rowNotNA)
  meanAbsEff  <- mean(abs(rowNotNA))
  sdEffect    <- sd(rowNotNA)
  CoefVar     <- sdEffect / meanEffect
  results <- list(N     = NumWithData, 
                  mnEf  = meanEffect, 
                  mnAbs = meanAbsEff, 
                  sdEf  = sdEffect, 
                  CV    = CoefVar)
}

# Get genome-wide file from emmax files
# First get effect size and abs(effect size) from one environment
EnvNames <- c("B1", "B3", "B4","B5","B6","E1","W1","W2","W3","W4")
base_name <- "BLUPs.txt_emmax.ps.mod" # the shared part of input file names
ThisEnvName <- EnvNames[1]
in_name <- str_c(ThisEnvName, base_name, sep = ".")
infile  <- file.path(path.expand("~"),"FlrGWAS","bash", "emmax.ps.mod files", in_name)
df      <- read_tsv(infile)
names(df) <- c("ChrNt", "EffectSize", "Pval", "Chr", "SNPpos")
df <- df %>% mutate(ChrNt      = as.character(ChrNt),
                    Chr        = as.character(Chr),
                    EffectSize = -10 * EffectSize, # QTL effects also were multipled by -10
                    abs.eff    = abs(EffectSize)) %>% 
            arrange(desc(abs.eff))
dftop <- df[1:NumSNPs,1:2] # Keep NumSNPs with largest absolute effects in this envirn
names(dftop) <- c("ChrNt", str_c(ThisEnvName, "Eff.Siz", sep = "."))

# Next get effect size and abs(effect size) from remaining environments
# and merge each envirn into data frame using full_join
for(this.env in 2:10) {
  ThisEnvName <- EnvNames[this.env]
  (in_name  <- str_c(ThisEnvName, base_name, sep = "."))
  infile    <- file.path(path.expand("~"),"FlrGWAS","bash", "emmax.ps.mod files", in_name)
  df        <- read_tsv(infile)
  names(df) <- c("ChrNt", "EffectSize", "Pval", "Chr", "SNPpos")
  df <- df %>% mutate(ChrNt      = as.character(ChrNt),
                      Chr        = as.character(Chr),
                      EffectSize = -10 * EffectSize, # QTL effects also were multipled by -10
                      abs.eff    = abs(EffectSize)) %>% 
               arrange(desc(abs.eff))
  df <- df[1:NumSNPs,1:2]  # Keep NumSNPs with largest absolute effects in this envirn
  names(df) <- c("ChrNt", str_c(ThisEnvName, "Eff.Siz", sep = "."))
  dftop <- full_join(dftop, df) # full_join, keep all
}

(NumTopSNPs <- nrow(dftop))

#-----------------------------------------------------

# Compute the number of environments with non-missing effect sizes
# Compute the mean of absolute value of effect sizes
dftop$N     <- 1
dftop$mnEf  <- NA
dftop$mnAbs <- NA
dftop$sdEf  <- NA
dftop$CV    <- NA
for(this.row in 1:nrow(dftop)) {
  results <- NumNotNA(dftop[this.row,])
  dftop$N[this.row]     <- as.numeric(results$N)
  dftop$mnEf[this.row]  <- as.numeric(results$mnEf)
  dftop$mnAbs[this.row] <- as.numeric(results$mnAbs)
  dftop$sdEf[this.row]  <- as.numeric(results$sdEf)
  dftop$CV[this.row]    <- as.numeric(results$CV)
}

dftop <- dftop %>% filter(N >= MinNumEnvs) # %>% arrange(desc(N), desc(mnAbs))
dim(dftop)

# Sort SNPs by position, and drop SNPs if they are too close
LGpos <- dftop$ChrNt
LG    <- substr(LGpos, 1, 1)
pos   <- as.numeric(substr(LGpos, 3, length(LGpos)))
# pos   <- as.numeric(pos) 
LGpos <- data.frame(LGpos, LG, pos, stringsAsFactors = FALSE)
LGpos <- arrange(LGpos, LG, pos)
LGpos <- LGpos %>% rename(ChrNt = LGpos) %>% mutate(keep = TRUE)
df.merge <- left_join(LGpos, dftop)

curr.LG  <- df.merge$LG[1]
curr.SNP <- df.merge$pos[1]
for(this.row in 2:nrow(df.merge)) {
  cond <- (df.merge$pos[this.row] - curr.SNP < Too.Close) & (curr.LG == df.merge$LG[this.row])
  if(cond) { # too close. Don't keep
    df.merge$keep[this.row] = FALSE 
  } else {
    curr.LG  <- df.merge$LG[this.row]
    curr.SNP <- df.merge$pos[this.row]
  }
}

df.merge <- df.merge %>% filter(keep) %>% select(-keep) %>% 
  arrange(desc(mnAbs))

# Exclude BLS regions: If on correct LG and within limits of BLS, then TRUE
BLS.cond1 <- ((df.merge$LG  == BLS$LG1[1]) &  # TRUE if in BLS region 1
                ((df.merge$pos >= BLS$LG1[2]) & (df.merge$pos <= BLS$LG1[3])))
BLS.cond2 <- ((df.merge$LG  == BLS$LG5[1]) &  # TRUE if in BLS region 2
                ((df.merge$pos >= BLS$LG5[2]) & (df.merge$pos <= BLS$LG5[3])))
BLS.cond3 <- ((df.merge$LG  == BLS$LG7[1]) &  # TRUE if in BLS region 3
                ((df.merge$pos >= BLS$LG7[2]) & (df.merge$pos <= BLS$LG7[3])))
BLS.cond.all <- BLS.cond1 | BLS.cond2 | BLS.cond3
# BLS.cond.all is a logical vector, TRUE for SNPs in the BLS region

# Next, drop rows for SNPs in BLS region:
df.merge <- df.merge[! BLS.cond.all, ]

if(nrow(df.merge) < 100) {
  print("ERROR: dataframe df.merge has less than 100 rows")
}
df.merge <- df.merge[1:100,] 
SNPnames <- df.merge$ChrNt
# df.merge holds 100 SNPs with as many environmental effect results as possible
# and as large absolute effect sizes as possible, without clustered SNP positions

#-----------------------------------------------------

# Get corrs among environmental effects for all pairs of SNPs
# based on pairwise correlations for rows of df.merge

e.names <- names(df.merge)[4:13] # Gets Env names 1 .. 10
# df.envs <- df.merge %>% select(e.names) # not using this

# Get data frame with correlations
cell <-  0
for(ii in 2:100) {
  for(jj in 1:(ii - 1)) {
    cell <- cell + 1
    Qii <- unlist(c(df.merge[ii, 4:13], use.names = FALSE))
    Qjj <- unlist(c(df.merge[jj, 4:13], use.names = FALSE))
    Q.ii.jj <- cor(Qii, Qjj, use = "pairwise.complete.obs")
    corr.ii.jj <-  c(cell, ii, jj, SNPnames[ii], SNPnames[jj], Q.ii.jj)
    if(cell == 1) {
      corr.info <- corr.ii.jj
    } else {
      corr.info <-  rbind(corr.info, corr.ii.jj)
    }
  }
}

df.corr <- as.tibble(corr.info)
names(df.corr) <- c("cell", "row.i", "row.j", "SNPi", "SNPj", "corr")
df.corr$corr <- as.numeric(df.corr$corr)
glimpse(df.corr)
(mean_corr_SNPs <- mean(df.corr$corr))

# Plot histogram of pairwise correlations of effect sizes for top 100 SNPs
ggplot(df.corr) + geom_histogram(aes(x = corr), bins = 100) +
  labs(x = "Distribution of correlated SNP effects across environments") +
  ylab("Frequency") +   xlim(-1, 1) + theme_classic()

# Input data with pairwise correlations of effect sizes for QTLs
in_name <- "QTL_Correlations.txt" 
in_name <- file.path(path.expand("~"),"FlrGWAS", "output", in_name)
df.QTL.corrs <- read_tsv(in_name)
(mean_corr_QTLs <- mean(df.QTL.corrs$Corr))

# Plot histogram of pairwise correlations of effect sizes for QTLs
ggplot(df.QTL.corrs) + geom_histogram(aes(x = Corr), bins = 100) +
  labs(x = "Distribution of correlated QTL effects across environments") +
  ylab("Frequency") +   xlim(-1, 1) +  theme_classic()

# Get density plots for QTLs & SNPs
df1 <- df.corr$corr
df1 <- data.frame(df1) %>% rename(Corr = df1) # SNPs
df2 <- df.QTL.corrs$Corr
df2 <- data.frame(df2) %>% rename(Corr = df2) # QTLs
ggplot() + geom_density(aes(x=Corr), colour="red", data=df1) + 
  geom_density(aes(x=Corr), colour="blue", data=df2)
# Distribution in red for SNPs, and in blue for QTLs

#-----------------------------------------------------
# Permutation test: does the average correlation among QTLs differ from the
# average correlation among SNPs?

Num.QTLs <- 6
num.SNPs.sampled <- Num.QTLs # draw this many SNPs, for comparison to QTLs
Num.perms <- 25000
Num.SNPs <- 100
SNPnames <- df.merge$ChrNt
num.corrs <- num.SNPs.sampled*(num.SNPs.sampled - 1) / 2
hold.perm.means <- rep(NA, Num.perms)

# Permutation comparison of the distribution of correlations for SNPs vs QTLs
for(this.perm in 1:Num.perms) {
  keep.rows <- sample(Num.SNPs, Num.QTLs)
  df.sample <- df.merge
  df.sample$toss <- TRUE
  for(item in keep.rows) {
    df.sample$toss[item] <- FALSE  
  }
  df.sample <- df.sample %>% filter(toss == FALSE) %>% select(-toss)
  dim(df.sample)
  hold.corrs <- rep(NA, num.corrs)
  cell <-  0
  for(ii in 2:num.SNPs.sampled) {
    for(jj in 1:(ii - 1)) {
      cell <- cell + 1
      Qii <- unlist(c(df.sample[ii, 4:13], use.names = FALSE))
      Qjj <- unlist(c(df.sample[jj, 4:13], use.names = FALSE))
      Q.ii.jj <- cor(Qii, Qjj, use = "pairwise.complete.obs")
      hold.corrs[cell] <- Q.ii.jj
    }
  }
  hold.perm.means[this.perm] <- mean(hold.corrs)
  # print(mean(hold.corrs))
}
hold.perm.means <- sort(hold.perm.means)
hold.perm.means[0.025  * Num.perms]
hold.perm.means[0.975 * Num.perms]
mean_corr_QTLs
mean(hold.perm.means)

# For observed mean correlation, where is it in permuted distribution?
percentile <- ecdf(hold.perm.means) # get CDF within null distribution
(p.value <- 2.0 * percentile(mean_corr_QTLs)) # two tailed test
# The mean cross-environment correlation between pairs of SNPs is significantly
# larger than the cross-environment correlation between pairs of QTLs


