# Using emmax output file, extract climate SNPs from PC2 & PC4. Find overlap
# with FlrDat SNPS. How many climate PC SNPs are near flowering time QTLs? 

# Thomas Mitchell-Olds, revised 2020-12-01

library(tidyverse)

half.width <- 100000 # Center on peak SNP, +/- half.width nt
Sig.thresh <- 8.0    # A constant, do not change

# Get climate PC SNP info
expt    <- c("PC2", "PC4")
in_name <- c("Ref_PC2.txt_emmax.ps.mod", "Ref_PC4.txt_emmax.ps.mod")

get.df <- function(in_name, expt){ # read in climate PC SNP info
  expt.folder <- str_c("_", expt) # add leading underscore for folder
  # Get genome-wide file from emmax
  infile  <- file.path(path.expand("~"),"FlrGWAS","bash", expt.folder, in_name)
  df      <- read_tsv(infile)
  names(df) <- c("ChrNt", "EffectSize", "Pval", "Chr", "SNPpos")
  df <- df %>% mutate(ChrNt = as.character(ChrNt),
                      Chr   = as.character(Chr),
                      logP  = -1.0*log10(Pval),
                      kind  = "Climate") 
  df <- df %>% filter(logP >= 8.0) %>% select(Chr, SNPpos, kind)
  return(df)
}
  
# input dataframe for significant climate PC SNPs
df1 <- get.df(in_name[1], expt[1]) 
df2 <- get.df(in_name[2], expt[2]) 
df.Clim <- rbind(df1, df2) %>% mutate(keep = FALSE)
dfC.len <- nrow(df.Clim)

# input dataframe for flowering time QTLs
QTL_infile <- file.path(path.expand("~"),"FlrGWAS","bash", "SNP_Locations.txt")
df.Q <- read_tsv(QTL_infile)
df.Q <- df.Q %>% mutate(Chr = as.character(Chr),
                        keep = FALSE) %>% 
                 select(Chr, SNPpos, kind, keep) 
dfQ.len <- nrow(df.Q)

# Data frame with climate PCs and flowering time QTLs
df.all <- rbind(df.Clim, df.Q)
df.all <- df.all %>% arrange(Chr, SNPpos)
dfA.len <- nrow(df.all)

for(ii in(1:dfQ.len)) { # for each QTL
  for(jj in(1:dfC.len)) { # for each climate SNP
    cond1 <- df.Q[ii, 1] == df.Clim[jj, 1] # same chromosomes?
    cond2 <- abs(df.Q[ii, 2] - df.Clim[jj, 2]) <= half.width # close?
    if(cond1 & cond2) {
      df.Clim[ii, 4] <- TRUE
    }
  }
}

nrow(df.Clim)
df.Clim.keep <- df.Clim %>% filter(keep)
nrow(df.Clim.keep)

# There are 169 significant climate PC SNPs. None of these are close (+/100 kb)
# to the nine flowering time QTLs that we found. Of the 169 significant for
# climate PC2 & PC4, 19 are close to a known flowering genes. Also, we found
# four significant GxE SNPs. Three of them are close to flowering genes.




